/**
 * @author mrdoob / http://mrdoob.com/
 */

THREE.DCLoader = function ( manager ) {

	this.manager = ( manager !== undefined ) ? manager : THREE.DefaultLoadingManager;

};

THREE.DCLoader.prototype = {

	constructor: THREE.DCLoader,

	load: function ( url, onLoad, onProgress, onError ) {

		var scope = this;

		var loader = new THREE.XHRLoader( scope.manager );
		loader.setCrossOrigin( this.crossOrigin );
		loader.load( url, function ( text ) {

			onLoad( scope.parse( text ) );

		} );

	},

	parse: function ( data ) {

		function pushMesh() {
			for(var k = 0 ; k < _geometry.vertices.length ; k+=3)
				_geometry.faces.push( new THREE.Face3( k, k+1, k+2 ) ); 
		
			_geometry.computeCentroids();
			_geometry.computeBoundingBox();
			for(var j = 0 ; j < _uniforms.length ; ++j) {
				if(_tex[0]) { _uniforms[j]["tex1"].value = _tex[0]; _uniforms[j]["texMat1"].value = _texMat[0]; }
				if(_tex[1]) { _uniforms[j]["tex2"].value = _tex[1]; _uniforms[j]["texMat2"].value = _texMat[1]; }
				if(_tex[2]) { _uniforms[j]["tex3"].value = _tex[2]; _uniforms[j]["texMat3"].value = _texMat[2]; }
				if(_tex[3]) { _uniforms[j]["tex4"].value = _tex[3]; _uniforms[j]["texMat4"].value = _texMat[3]; }
			
				var material = new THREE.ShaderMaterial({
				  uniforms: _uniforms[j],
				  attributes: _attributes,
				  vertexShader: document.getElementById('vertexShader').textContent,
				  fragmentShader: document.getElementById('fragmentShader').textContent,
				  transparent: true,
				  depthWrite: false,
				  depthTest: false,
				  side: THREE.DoubleSide
				});
				var mesh = new THREE.Mesh( _geometry, material );
				mesh.name = _name;
				_group.add( mesh );
			}
		}
	
		var _width, _height;
		
		var _group = new THREE.Object3D();
		
		var _name = null;
		var _geometry = null;
		var _uniforms = null;
		var _attributes = null;
		var _tex = null;
		var _texMat = null;
		
		var Uniforms = function() {
			this["mvMatrix"] = { type: "m4", value: null };
			this["scaleColor"] = { type: "f", value: 1.0 };
			this["scaleOpacity"] = { type: "f", value: 1.0 };
		//uniform bool opacity;
		//uniform bool triangulation;

			this["tex1"] = { type: "t", value: null };
			this["texMat1"] = { type: "m4", value: new THREE.Matrix4() };
			this["tex2"] = { type: "t", value: null };
			this["texMat2"] = { type: "m4", value: new THREE.Matrix4() };
			this["tex3"] = { type: "t", value: null };
			this["texMat3"] = { type: "m4", value: new THREE.Matrix4() };
			this["tex4"] = { type: "t", value: null };
			this["texMat4"] = { type: "m4", value: new THREE.Matrix4() };
		};
		
		var Attributes = function() {
			this["m_I"] = { type: "v4", value: [] };
			this["m_wI"] = { type: "v4", value: [] };
			this["m_A"] = { type: "v4", value: [] };
		};

		// wh int int
		var size_pattern = /^wh( [\d]+)( [\d]+)/;
		// v float float float
		var vertex_pattern = /^v( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)/;
		var colorI_pattern = /^I( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)/;
		var colorW_pattern = /^wI( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)/;
		var colorA_pattern = /^A( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)/;
		// mvMatrix float float float ...
		var mvMatrix_pattern = /^mvMatrix( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)/;
		
		var tex_pattern =  /^tex (.+)/;
		var texMat_pattern = /^texMat( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)( +[\d|\.|\+|\-|e]+)/;

		var lines = data.split( "\n" );

		for ( var i = 0; i < lines.length; i ++ ) {

			var line = lines[ i ];
			line = line.trim();

			var result;

			if ( line.length === 0 || line.charAt( 0 ) === '#' ) {

				continue;

			} else if ( ( result = vertex_pattern.exec( line ) ) !== null ) {

				_geometry.vertices.push( new THREE.Vector3(
					parseFloat( result[ 1 ] ),
					parseFloat( result[ 2 ] ),
					0
				) );

			} else if ( ( result = colorI_pattern.exec( line ) ) !== null ) {

				_attributes["m_I"].value.push( new THREE.Vector4(
						parseFloat( result[ 1 ] ),
						parseFloat( result[ 2 ] ),
						parseFloat( result[ 3 ] ),
						parseFloat( result[ 4 ] )
					)
				);
				
			} else if ( ( result = colorW_pattern.exec( line ) ) !== null ) {

				_attributes["m_wI"].value.push( new THREE.Vector4(
						parseFloat( result[ 1 ] ),
						parseFloat( result[ 2 ] ),
						parseFloat( result[ 3 ] ),
						parseFloat( result[ 4 ] )
					)
				);
				
			} else if ( ( result = colorA_pattern.exec( line ) ) !== null ) {

				_attributes["m_A"].value.push( new THREE.Vector4(
						parseFloat( result[ 1 ] ),
						parseFloat( result[ 2 ] ),
						parseFloat( result[ 3 ] ),
						parseFloat( result[ 4 ] )
					)
				);

			} else if ( ( result = size_pattern.exec( line ) ) !== null ) {

				// ["wh 1024 768", "1024", "512"]
				_width = parseInt( result[ 1 ] );
				_height = parseInt( result[ 2 ] );

			} else if ( ( result = mvMatrix_pattern.exec( line ) ) !== null ) {

				// ["mvMatrix 0 0 0 0 0 0 0 0 0 0", "1024", "512"]
				var mvMatrix = new THREE.Matrix4( 
					parseFloat( result[ 1 ] ), parseFloat( result[ 4 ] ), 0, parseFloat( result[ 7 ] ),  
					parseFloat( result[ 2 ] ), parseFloat( result[ 5 ] ), 0, parseFloat( result[ 8 ] ),  
					parseFloat( result[ 3 ] ), parseFloat( result[ 6 ] ), 1, 0,
					0,0,0,1
				);
				var instance = new Uniforms();
				instance["mvMatrix"].value = mvMatrix;
				_uniforms.push(instance);
				
			} else if ( ( result = tex_pattern.exec( line ) ) !== null ) {

				var texture = THREE.ImageUtils.loadTexture( "data/"+result[1] );
				texture.generateMipmaps = false;
				texture.minFilter = THREE.NearestFilter;
				_tex.push( texture );
				
			} else if ( ( result = texMat_pattern.exec( line ) ) !== null ) {

				_texMat.push( new THREE.Matrix4( 
					parseFloat( result[ 1 ] ), parseFloat( result[ 2 ] ), 0, parseFloat( result[ 7 ] ),  
					parseFloat( result[ 4 ] ), parseFloat( result[ 5 ] ), 0, parseFloat( result[ 8 ] ),  
					0, 0, 1, 0,
					parseFloat( result[ 3 ] ), parseFloat( result[ 6 ] ), 0, 1
					)
				);
				
			} else if ( /^o /.test( line ) ) {

				if(_geometry) pushMesh();

				_name = line.substring( 2 ).trim();
				
				_geometry = new THREE.Geometry();
				_uniforms = new Array();
				_attributes = new Attributes();
				_tex = new Array();
				_texMat = new Array();

			} else {

				//console.log( "THREE.OBJLoader: Unhandled line " + line );

			}

		}
		
		if(_geometry) pushMesh();

		return { dc: _group, w: _width, h: _height};

	}

};
